; ------------------------------------------------------------------------------
; The following is a port of "Killer Bees", David Rolfe's trivial Intellivision 
; game to the ASM1600 assembler.  David's original source code can be found 
; here:
;
; http://home.earthlink.net/~davidrolfe/trivia.htm
;
; Killer Bees was original written in APh MX1600 assembler.  In addition to 
; some minor changes required by differences in the assembler syntax it has 
; been necessary to reverse engineer some constants and macros originally 
; provided by include files within the APh framework.  These are listed first, 
; followed by a minimally changed version of David's code.  Additional comments 
; have been added.  Helpfully David exclusively used upper case, therefore, to
; distinguish my additions from his original I've used mixed case comments.
;
;
; Enjoy!
;
; decle
;

    ROMW    10


; External constants - the following constants will have been defined in the 
; SWELL include file.  Values have been taken from  or inferred from Your 
; Friend the EXEC (YFTE) or reverse engineerred

; MOB constants

.OBJBASE        EQU     $031d           ; base address of MOB definitions
.OBJSTAT        EQU     $0              ; see YFTE page 4-2
.OBJDAT         EQU     $1              ;
.OBJX           EQU     $2              ;
.OBJY           EQU     $3              ;
.OBJMOT         EQU     $4              ;
.OBJSEQ         EQU     $5              ;    
.OBJTIM         EQU     $6              ;
.OBJXTR         EQU     $7              ;


; MOB database status constants

ST.BLUE         EQU     $0001           ; see YFTE page 4-3
ST.RED          EQU     $0002           ;
ST.GREEN        EQU     $0004           ;
ST.XTRA         EQU     $0008           ;
ST.PASTEL       EQU     $0010           ;
ST.BACK         EQU     $0020           ;
ST.YRES         EQU     $0040           ;
ST.YFULL        EQU     $0080           ;
ST.YSIZE        EQU     $0100           ;
ST.XMIRROR      EQU     $0200           ;
ST.YMIRROR      EQU     $0400           ;
ST.INTERACT     EQU     $0800           ;
ST.VISIBLE      EQU     $1000           ;
ST.XSIZE        EQU     $2000           ;
ST.XTR1         EQU     $4000           ;
ST.STOP         EQU     $8000           ;


; OFF event constants - see YFTE page 4-6

OF.CENTER       EQU     3               ; 
OF.SCREEN       EQU     1               ; 


; Screen constants

BACKTAB         EQU     $200            ; address of backtab
BACKX           EQU     20              ; characters per backtab row
BACKY           EQU     12              ; rows in backtab
FOREX           EQU     167             ; MOB X resolution
FOREY           EQU     103              ; MOB Y resolution
CH.RED          EQU     $2              ; foreground colours
CH.GREEN        EQU     $4              ; 
CH.WHITE        EQU     $7              ;



; Input constants - evaluated from documentation in YFTE and by observing 
; memory locations in 8 and 16 bit RAM written to by the EXEC and the
; influence of the keypad on the values written  

.KEYPAD         EQU     $11f            ; left controller keypad input address



; EXEC function entry points - taken from the source code of dis1600
; and examining the disassembly of the EXEC.  Within code libraries the order
; of function documentation within YFTE broadly matches the object code.

DETSTPOB        EQU     $12bc           ; see YFTE page E-11
GETVEL          EQU     $1629           ; see YFTE page E-6
INITOBJS        EQU     $16b4           ; see YFTE page E-7
FILLRAM         EQU     $1741           ; see YFTE page E-9
INLMES          EQU     $187b           ; see YFTE page E-2



; External macros - the following macros will have been defined in the MACROS
; include file.  These have been reverse engineered from documentation in YFTE,
; the annotated disassembly of Utopia completed by JoeZ and examining 
; disassemblies of other early games.  They appear to work in this context, but
; are unlikely to be comprehensive.  For more information see YFTE

MACRO INTDEF back, mob
        DECLE   %back% + (%mob% * 2)
        ENDM

MACRO INTERACT mob, addr
        DECLE   ((%mob% AND 2) * $100) + ((%mob% AND 4) * $40) + (%addr% AND $ff), (%addr% / $100) + ((%mob% AND 1) * $100)
        ENDM

MACRO OFF field, addr
        DECLE   (%addr% AND $ff) + (%field% * $100),  %addr% / $100
        ENDM

MACRO POSITION val
        DECLE   $00, %val%
        ENDM

MACRO PROCESS addr, count
        BIDECLE   %addr%, %count%
        ENDM

MACRO SEQDEF pict, count
        DECLE   %count%, %pict%
        ENDM

MACRO SEQRATE rate
        BIDECLE   %rate%
        ENDM


; Game specific constants - these were computed with special assembler magic 
; in David's original

MANPCT          EQU     0               ; picture number of first man animation
MINEPCT         EQU     16              ; picture number of mine



        ORG $5000


; ------------------------------------------------------------------------------
; From here on is broadly David's code with minimal changes



;       TITLE   "TRIVIA -- DEMO PROGRAM"

;       INSERT  "SWELL"
;       INSERT  "MACROS"

;       SEXTERN .OBJBASE                ;EXEC
;       SEXTERN .KEYPAD                 ;KEYPAD
;       EXTERN  FILLRAM,INITOBJS,GETVEL ;ESUBS
;       EXTERN  DETSTPOB                ;OBJECT
;       EXTERN  INLMES                  ;IO
;       EXTERN  PIANO                   ;SOUND

;       MAN=.OBJBASE+8                  ;MAN OBJECT
;       MINE=.OBJBASE+16                ;MINE OBJECT

MAN     EQU .OBJBASE+8                  ;MAN OBJECT
MINE    EQU .OBJBASE+16                 ;MINE OBJECT

PSECT
        BIDECLE MOVGRAM                 ;MOVING PICTURE BASE
        BIDECLE CNTERS                  ;PROCESS TABLE
        BIDECLE RESET                   ;RESET ROUTINE
        BIDECLE BAKGRAM                 ;BACKGROUND PICTURE BASE
        BIDECLE BLIST                   ;BACKGROUND LIST
        BIDECLE TNAME                   ;TRIVIA NAME
        DECLE   %10000001               ;KEYPAD CLICKS
        DECLE   0                       ;NO EXTENDED BORDER
        DECLE   0                       ;NORMAL MODE
        DECLE   CH.GREEN,0,0,0          ;COLOR STACK
        DECLE   CH.RED                  ;BORDER COLOR
        DECLE   0                       ;NOTHING
        DECLE   62*8                    ;RESERVE SPACE IN GRAM FOR MAN
        DECLE   61*8                    ;MINE


CNTERS  PROCESS CTRL,2                  ;CONTROL FUNCTIONS
        PROCESS 0,0                     ;TERMINATE LIST


;BLIST  DECLE   ..NUMPCT                ; this magic is described on page C-1
;       .NUMPCT.==0                     ; of YFTE
;       GM.CHAR "A"-" ",YMIR!GRO,1      ;
;       ..NUMPCT=.NUMPCT.               ;

BLIST   DECLE   1                       ; number of GRAM cards to init
        DECLE   $0143, $0010            ; valid values pulled from examples



;       TITLE   "SPECIAL TITLE"

TNAME   DECLE   78,"KILLER BEES",0      ;YEAR AND GAME

DOTITL  BEGIN
        MVII    #CH.WHITE,R3            ;WRITE IN WHITE
        MVII    #BACKTAB+3*BACKX+1,R4   ;ON SCREEN
        CALL    INLMES                  ;WRITE MESSAGE
        DECLE   " TALKIN HORSE INC ",0  ;
        RETURN                          ;



;       TITLE   "RESET ROUTINE"

RESET   BEGIN
        MVII    #BACKTAB,R4             ;BACKGROUND TABLE
        MVII    #240,R0                 ;COUNT
        CLRR    R1                      ;FILL WITH ZERO
        CALL    FILLRAM                 ;FILL RAM
        MVII    #MAN,R2                 ;DATABASE OF OBJECT
        SDBD
        MVII    #MANRAM,R1              ;PROTOTYPE DATABASE
        MVII    #2,R0                   ;NUMBER OF OBJECTS
        CALL    INITOBJS                ;INITIALIZE HIM
        RETURN                          ;READY TO GO



;       TITLE   "DO CONTROL OF GUY"

CTRL    BEGIN
        MVI     MAN+.OBJSEQ,R3          ;CLEAR SPEED OF SEQUENCING
        ANDI    #$ffff XOR $03F0,R3     ; ..
        MVI     .KEYPAD,R1              ;WHERE DOES HE WANT TO GO?
        MOVR    R1,R0                   ;CHECK STATUS BITS
        ANDI    #$C0,R0                 ; ..
        BNZ     STPGUY                  ;STOP MOTION IF WHEEL NOT PUSHED
        XORI    #$03F0,R3               ;MOVING, SO HE SEQUENCES AT MAX RATE
        MVO     R3,MAN+.OBJSEQ          ; ..
        MVII    #12,R0                  ;SPEED AT WHICH HE RUNS
        CALL    GETVEL                  ;FIGURE VELOCITY
        MVO     R0,MAN+.OBJMOT          ;AND STORE IT
        MVI     MAN+.OBJSTAT,R1         ;GET STATUS BITS
        ANDI    #$ffff XOR ST.XMIRROR,R1;NOT MIRRORED IN X
        TSTR    R0                      ;FACING PROPERLY?
        BPL     CTRL1                   ;GO IF YES
        XORI    #ST.XMIRROR,R1          ;NO, MUST MIRROR IN X
CTRL1   MVO     R1,MAN+.OBJSTAT         ;AND RESTORE STATUS
        RETURN                          ;AND RETURN


STPGUY  MVO     R3,MAN+.OBJSEQ          ;STOP SEQUENCING
        CLRR    R0                      ;AND MOTION
        MVO     R0,MAN+.OBJMOT          ; ..
        RETURN                          ;AND RETURN



;       TITLE   "MAN STARTS TO RUN OFF SCREEN EDGE"

CENTER  BEGIN

; The following looks to be a mechanism to comment out code

;REPEAT 0
;       MVII    #FOREX/2,R0             ;POSITION IN MIDDLE OF SCREEN
;       SWAP    R0                      ; ..
;       MVO     R0,MAN+.OBJX            ; ..
;       MVII    #FOREY/2,R0             ;SAME FOR Y
;       SWAP    R0                      ; ..
;       MVO     R0,MAN+.OBJY            ; ..
;ENDR

        SLL     R2,1                    ;OFFSET
        ADDR    R2,R7                   ;DISPATCH
        B       OFFRIT
        B       OFFLFT
        B       OFFBOT
;       B       OFFTOP                  ; interesting code as comment style

OFFTOP  MVII    #FOREY,R0               ; flip to bottom of screen
        SWAP    R0                      ;
        INCR    R7                      ; skip next instruction

OFFBOT  CLRR    R0                      ; flip to top of screen
        MVO     R0,MAN+.OBJY            ; set Y position
        RETURN                          ; done

OFFLFT  MVII    #FOREX,R0               ; flip to right of screen
        SWAP    R0                      ;
        INCR    R7                      ; skip next instruction

OFFRIT  CLRR    R0                      ; flip to left of screen
        MVO     R0,MAN+.OBJX            ; set X position
        RETURN                          ; done


HITMINE BEGIN
        MVII    #$0808,R4               ;HOW CLOSE WE CAN GET TO MINE
;        CALL    DETSTPOB                ;STOP COLLISION
        MVI@    R1,R0                   ;STATUS OF MAN
        ANDI    #$ffff XOR ST.RED,R0    ;SET RED ALSO
        XORI    #ST.RED,R0              ; ..
        MVO@    R0,R1                   ;AND RESTORE
        ADDI    #.OBJTIM,R1             ;TIME WHEN DONE
        MVII    #120,R0                 ;2 SECONDS
        MVO@    R0,R1                   ;STORE
        RETURN                          ;AND QUIT


HITDONE BEGIN                           ;TIMER EXPIRES
        MVI@    R1,R0                   ;CLEAR READ
        ANDI    #$ffff XOR ST.RED,R0    ; ..
        MVO@    R0,R1                   ;AND RESTORE
        RETURN                          ;AND RETURN



;       TITLE   "PROTOTYPE DATA BLOCK"

MANRAM  BIDECLE ST.VISIBLE OR ST.INTERACT OR ST.YRES OR ST.BLUE         ;BLUE MAN
        BIDECLE MANROM                  ;ROM DATA BLOCK
        POSITION FOREX/2                ;INITIALIZE IN CENTER
        POSITION FOREY/2                ; ..
        DECLE   0,0                     ;NO INITIAL SPEED
        SEQRATE 0                       ;DOES NOT SEQUENCE INITIALLY
        BIDECLE 0                       ;NO INITIAL TIME-OUT
        BIDECLE 0                       ;NO EXTRA WORD


MINERAM BIDECLE ST.VISIBLE OR ST.INTERACT OR ST.YFULL OR ST.RED         ;MINE
        BIDECLE MINEROM                 ;ROM DATA BLOCK
        POSITION FOREX-30               ;INITIALIZE TO RIGHT
        POSITION FOREY/2                ;IN CENTER
        DECLE   0,0                     ;NO SPEED
        SEQRATE 0                       ;NO SEQUENCE
        BIDECLE 0                       ;NO TIME OUT
        BIDECLE 0                       ;NO EXTRA WORD


MANROM  OFF     OF.SCREEN,CENTER        ;ROUTINE TO CALL WHEN GUY STEPS OFF
        BIDECLE HITDONE                 ;TIME OUT
        SEQDEF  MANPCT,8                ;STARTS FROM FIRST PICTURE IN GRAM
        INTDEF  0,1                     ;CAN HIT MINE
        INTERACT 2,HITMINE              ;WHERE TO GO WHEN HIT

MINEROM OFF     OF.CENTER,0             ;WILL NOT LEAVE SCREEN
        BIDECLE 0                       ;NO TIME OUT
        SEQDEF  MINEPCT,1               ;ONLY 1 PICTURE
        INTDEF  0,0                     ;DOESN'T REGISTER COLLISION


;       TITLE   "RUNNING MAN PICTURES"
 
MOVGRAM 

;       MANPCT=(*-MOVGRAM)/8            ; magic replaced by constant above

        DECLE   $3ff XOR $F3,$3ff XOR $F1,$3ff XOR $FF,$3ff XOR $F3,$3ff XOR $F3,$3ff XOR $F3,$3ff XOR $F3,$3ff XOR $F1,$3ff XOR $F3,$3ff XOR $F3,$3ff XOR $F3,$3ff XOR $C3,$3ff XOR $D7,$3ff XOR $D7,$3ff XOR $F7,$3ff XOR $F3
        DECLE   $3ff XOR $F9,$3ff XOR $F8,$3ff XOR $FF,$3ff XOR $F3,$3ff XOR $E3,$3ff XOR $E3,$3ff XOR $E1,$3ff XOR $E7,$3ff XOR $E3,$3ff XOR $C1,$3ff XOR $DD,$3ff XOR $D1,$3ff XOR $D7,$3ff XOR $DF,$3ff XOR $DF,$3ff XOR $EF
        DECLE   $3ff XOR $F9,$3ff XOR $F8,$3ff XOR $FF,$3ff XOR $E3,$3ff XOR $C3,$3ff XOR $D1,$3ff XOR $D3,$3ff XOR $FF,$3ff XOR $E3,$3ff XOR $C0,$3ff XOR $DE,$3ff XOR $D8,$3ff XOR $DB,$3ff XOR $BB,$3ff XOR $BF,$3ff XOR $BF
        DECLE   $3ff XOR $F3,$3ff XOR $F1,$3ff XOR $FF,$3ff XOR $87,$3ff XOR $A7,$3ff XOR $A0,$3ff XOR $E7,$3ff XOR $FF,$3ff XOR $E7,$3ff XOR $C0,$3ff XOR $DE,$3ff XOR $9D,$3ff XOR $BD,$3ff XOR $3F,$3ff XOR $7F,$3ff XOR $7F
        DECLE   $3ff XOR $F3,$3ff XOR $F1,$3ff XOR $8F,$3ff XOR $A7,$3ff XOR $A0,$3ff XOR $E7,$3ff XOR $E7,$3ff XOR $FF,$3ff XOR $E7,$3ff XOR $C0,$3ff XOR $DE,$3ff XOR $9E,$3ff XOR $BE,$3ff XOR $3F,$3ff XOR $7F,$3ff XOR $FF
        DECLE   $3ff XOR $F3,$3ff XOR $F1,$3ff XOR $FF,$3ff XOR $87,$3ff XOR $A7,$3ff XOR $A0,$3ff XOR $E7,$3ff XOR $FF,$3ff XOR $E7,$3ff XOR $C3,$3ff XOR $DB,$3ff XOR $99,$3ff XOR $3D,$3ff XOR $7D,$3ff XOR $FC,$3ff XOR $FF
        DECLE   $3ff XOR $F9,$3ff XOR $F8,$3ff XOR $FF,$3ff XOR $E3,$3ff XOR $C3,$3ff XOR $D1,$3ff XOR $D3,$3ff XOR $FF,$3ff XOR $C7,$3ff XOR $C3,$3ff XOR $DB,$3ff XOR $1B,$3ff XOR $7B,$3ff XOR $7B,$3ff XOR $FB,$3ff XOR $F9
        DECLE   $3ff XOR $F9,$3ff XOR $F8,$3ff XOR $FF,$3ff XOR $F3,$3ff XOR $E3,$3ff XOR $E3,$3ff XOR $E1,$3ff XOR $E7,$3ff XOR $E7,$3ff XOR $E7,$3ff XOR $E7,$3ff XOR $87,$3ff XOR $B7,$3ff XOR $B7,$3ff XOR $F7,$3ff XOR $F3

;       MINEPCT=(*-MOVGRAM)/8           ; magic replaced by constant above

        DECLE   $AA,$55,$AA,$55,$AA,$55,$AA,$55


BAKGRAM:DECLE   $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF         ;FULL PICTURE
